# coding=utf-8
"""
Holds current state of the mission manager
"""


# class MissionManagerOperation:
#
#     def __init__(self, operation_name):
#         self.name = operation_name
#
#
# INIT = MissionManagerOperation('init')
# LOADING = MissionManagerOperation('loading')
# LOADED = MissionManagerOperation('loaded')
#
#
# class MissionManagerState:
#     """
#     Holds current state of the mission manager
#     """
#
#     current_operation = INIT
#     current_context = None
