# coding=utf-8
"""
Manages all things related to UR
"""

from ._ur_voice_service_settings import URVoiceServiceSettings
from ._ur_voice_service import URVoiceService
from ._ur_install_dir import discover_ur_install_path
from ._ur_object import Airfield, URCoord, URFrequency
from ._ur_status import Status
