cfg =
{
    ["isPublic"] = true,
    ["missionList"] =
    {
        [1] = "{{ mission_file_path }}",
    }, -- end of ["missionList"]
    ["bind_address"] = "",
    ["port"] = "10308",
    ["advanced"] =
    {
        ["event_Role"] = false,
        ["allow_ownship_export"] = true,
        ["allow_object_export"] = true,
        ["pause_on_load"] = false,
        ["event_Connect"] = true,
        ["event_Ejecting"] = false,
        ["event_Kill"] = false,
        ["event_Takeoff"] = false,
        ["pause_without_clients"] = false,
        ["client_outbound_limit"] = 0,
        ["event_Crash"] = false,
        ["client_inbound_limit"] = 0,
        ["resume_mode"] = 1,
        ["allow_sensor_export"] = true,
    }, -- end of ["advanced"]
    ["password"] = "{{ passwd }}",
    ["require_pure_clients"] = false,
    ["version"] = 1,
    ["description"] = "",
    ["name"] = "{{ name }}",
    ["listLoop"] = false,
    ["listShuffle"] = false,
    ["maxPlayers"] = {{ max_players }},
} -- end of cfg
