# coding=utf-8
"""
Globally available commands
"""

# noinspection PyUnresolvedReferences
from esst.discord_bot.commands import DISCORD
# noinspection PyUnresolvedReferences
from esst.listener.commands import LISTENER
# noinspection PyUnresolvedReferences
from esst.dcs.commands import DCS
# noinspection PyUnresolvedReferences
from esst.server.commands import SERVER
